/* ISOAPP.C
 * ISO-14230 Box Application-Specific Routines
 * 3/15/02 Prune
 */
#define ISOAPPMAIN

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "isobox.h"
#include "isoapp.h"
#include "rs232.h"

extern unsigned char EnableTesterPresent;

void QueryStatus(void) 
{ 
unsigned char retry; 
unsigned int resp;
    for (retry=1; retry<=NUM_RETRIES; retry++) { 
        SendCommand('?'); 
	if (RecvInitResp(&resp,RESP_STATUS)>=0) break; 
    } 
    if (retry<=5) { 
	printf("\nStatus Code = %02X (%s)\n", resp,GetErrorCode(resp)); 
    } else printf(NotResp); 
} 

void ClearBuffers(void) 
{ 
unsigned char retry; 
unsigned int resp; 
    for (retry=1; retry<=NUM_RETRIES; retry++) { 
        SendCommand('X'); 
	if (RecvInitResp(&resp,RESP_STATUS)>=0) break; 
    } 
    if (retry<=5) { 
	if (resp==STATUS_OK) { 
	    printf("\nISO-14230 Box Buffers Cleared.\n"); 
	} else { 
	    printf("\nError Code = %02X (%s)\n", resp,GetErrorCode(resp)); 
	} 
    } else printf(NotResp); 
} 

void QueryKeyword(void)
{
unsigned char retry;
unsigned int resp;
    for (retry=1; retry<=5; retry++) {
        SendCommand('K');
	if (RecvInitResp(&resp,RESP_KEYWORD)>=0) break;
    }
    if (retry<=5) {
	printf("\nMost Recent Keyword = %04X\n", resp);
    } else printf(NotResp);
} 

void QueryRevision(void)
{
unsigned char retry;
unsigned int resp;
    for (retry=1; retry<=5; retry++) {
        SendCommand('R');
	if (RecvInitResp(&resp,RESP_REVISION)>=0) break;
    }
    if (retry<=5) {
	printf("\nFirmware Revision Level = %02X\n", resp);
	if (resp<3) {
	    printf("This rev has a bug in receive fast init response!\n");
	}
    } else printf(NotResp);
} 

void SetLineVoltage(void)
{
unsigned char retry;
unsigned int resp;
unsigned char i,cks;
    ISOarg[0]='V';
reask:
    printf("\nLine Voltage Menu\n\n(1) Auto-detect\n(2) 12V\n(3) 24V\n"
	"(4) Quit\n\nEnter choice (1-4): ");
    gets(string);
    switch (atoi(string)) {
	case 1: ISOarg[1]=0; break;
	case 2: ISOarg[1]=12; break;
	case 3: ISOarg[1]=24; break;
	case 4: return;
	default:
	    printf("\nInvalid option.\n");
	    goto reask;
    }
    for (retry=1; retry<=5; retry++) {
	SendMultiByteCommand(2);
	if (RecvInitResp(&resp,RESP_STATUS)>=0) break;
    }
    if (retry<=5) {
	if (resp==STATUS_OK) {
	    printf("\nLine Voltage Command Sent.\n");
	} else {
	    printf("\nError Code = %02X\n", resp, GetErrorCode(resp));
	}
    } else printf(NotResp);
} 

int SlowInit(void)
{
reask2:
    printf("(5) baud init or (C)arb init? ");
    gets(string);
    strupr(string);
    Method=string[0];
    if ((Method!='5')&&(Method!='C')) {
	printf("Type either 5 or C.\n");
	goto reask2;
    }
    if (GetParameters()<0) return(-1);
    return(Initialize());
}

int FastInit(void)
{
    Method='F';
    if (GetParameters()<0) return(-1);
    return(Initialize());
}

int Initialize(void)
{
unsigned int resp;
unsigned char retry,cks,i;
    ISOarg[0]=Method;
    ISOarg[1]=Addressing;
    ISOarg[2]=Source;
    ISOarg[3]=Target;
    for (retry=1; retry<=5; retry++) {
	SendMultiByteCommand(4);
        if (RecvInitResp(&resp,RESP_STATUS)>=0) break;
    }
    if (retry<=5) {
	if (resp==STATUS_OK) {
            printf("\nInitialized\n");
	    QueryKeyword();
	    handshake(RTS|DTR);	
	    return(0);
	} else {
	    printf("\nError Code = %02X (%s)\n", resp,GetErrorCode(resp));
	}
    } else printf(NotResp);
    return(-1);
}

int GetParameters(void)
{
int res;
    printf("\nEnter Source Address: ");
reaska:
    gets(string);
    if ((res=GetHex(&Source))<0) return(-1);
    if (!res) goto reaska;
    printf("\nEnter Target Address: ");
reaskb:
    gets(string);
    if ((res=GetHex(&Target))<0) return(-1);
    if (!res) goto reaskb;
reask3:
    printf("\nEnter (P)hysical addressing or (F)unctional addressing: ");
    gets(string);
    strupr(string);
    Addressing=string[0];
    if ((Addressing!='P')&&(Addressing!='F')) {
	printf("\nType either P or F.\n");
	goto reask3;
    }
reask10:
    printf("\nEnable Tester Present (Y/N)? ");
    gets(string);
    strupr(string);
    if ((string[0]!='Y')&&(string[0]!='N')) {
	printf("\nType either Y or N.\n");
        goto reask10;
    }
    if (string[0]=='Y') EnableTesterPresent=1;
    else EnableTesterPresent=0;
    return(0);
}

int GetHex(unsigned char *hexvalptr)
{
reask4:
    if (strlen(string)!=2) {
	if (string[0]==0) return(-1);
	printf("\nPlease enter only 2 hex digits or ENTER by itself to QUIT.");
	printf("\n\nRe-enter: ");
        return(0);
    }
    strupr(string);
    if ((string[0]>='0')&&(string[0]<='9')) {
	*hexvalptr=(string[0]-'0');
    } else {
	if ((string[0]>='A')&&(string[0]<='F')) {
	    *hexvalptr=((string[0]-'A')+10);
	} else {
	    printf("\nFirst digit was not hex.\nRe-enter: ");
	    return(0);
	}
    }
    (*hexvalptr)<<=4;
    if ((string[1]>='0')&&(string[1]<='9')) {
	*hexvalptr|=(string[1]-'0');
    } else {
	if ((string[1]>='A')&&(string[1]<='F')) {
	    *hexvalptr|=((string[1]-'A')+10);
	} else {
	    printf("\nSecond digit was not hex.\nRe-enter: ");
	    return(0);
	}
    }
    return(1);
}

