/* ISOBOX.C
 * ISO-14230 box primitives
 * 3/15/02 Prune
 */
#define ISOMAIN
#include "isobox.h"
#include "rs232.h"
#include "isoerr.h"

const unsigned char header[6]="14230";
const unsigned char respheader[6]="RESP:";
const unsigned char NotResp[]="\nISO-14230 Box is Not Responding!\n";

int RecvInitResp(unsigned int *respptr, RespEnum RespType)
{
int data;
unsigned char resphdridx,cks;
    cks=0;
    resphdridx=0;
    while (1) {
        if ((data=MyReadCharTimed(5000))>=0) {
	    if (data==respheader[resphdridx]) {
		cks+=data;
	        resphdridx++;
	        if (resphdridx>=5) break;
	    } else {
		if (data==respheader[0]) {
		    cks=data;
		    resphdridx=1;
		} else {
		    cks=0;
		    resphdridx=0;
		}
	    }
	} else return(-1);
    }
    switch (RespType) {
	case RESP_STATUS:
	case RESP_REVISION:
            if ((data=MyReadCharTimed(5000))>=0) {
		*respptr=data;
		cks+=data;
	    } else return(-1);
	    break;
	case RESP_KEYWORD:
            if ((data=MyReadCharTimed(5000))>=0) {
		*respptr=data;
		cks+=data;
	    } else return(-1);
	    (*respptr)<<=8;
            if ((data=MyReadCharTimed(5000))>=0) {
		*respptr|=data;
		cks+=data;
	    } else return(-1);
	    break;
    }	
    if ((data=MyReadCharTimed(5000))>=0) {
	if (data!=cks) return(-1);
    } else return(-1);
    if ((data=MyReadCharTimed(5000))>=0) {
	if (data!=(255-cks)) return(-1);
    } else return(-1);    
    return(0);
}

int SendCommand(unsigned char x)
{
unsigned char i,cks;
    for (cks=0, i=0; i<5; i++) { MyWriteChar(header[i]); cks+=header[i]; }
    MyWriteChar(x); cks+=x;
    MyWriteChar(cks); MyWriteChar(0xFF-cks);
    return(0);
}

int SendMultiByteCommand(unsigned char len)
{
unsigned char i,cks;
    if (len>10) return(-1);
    for (cks=0, i=0; i<5; i++) { MyWriteChar(header[i]); cks+=header[i]; }
    for (i=0; i<len; i++) { MyWriteChar(ISOarg[i]); cks+=ISOarg[i]; }
    MyWriteChar(cks); MyWriteChar(0xFF-cks);
    return(0);
}

char *GetErrorCode(unsigned char code)
{
    switch (code) {
	case ERROR_FAILED_BRING_K_AND_L_LOW:
	    return("Failed to assert K and L lines low");
	case ERROR_FAILED_BRING_K_LOW: 
	    return("Failed to assert K line low");
	case ERROR_FAILED_BRING_L_LOW: 
	    return("Failed to assert L line low");
	case ERROR_K_AND_L_WERE_STUCK_LOW: 
	    return("Failed to deassert K and L lines high");
	case ERROR_K_WAS_STUCK_LOW: 
	    return("Failed to deassert K line high");
	case ERROR_L_WAS_STUCK_LOW:
	    return("Failed to deassert L line high");
	case ERROR_K_AND_L_ARE_STUCK_LOW: 
	    return("K and L lines are shorted to ground");
	case ERROR_K_IS_STUCK_LOW: 
	    return("K line is shorted to ground");
	case ERROR_L_IS_STUCK_LOW: 
	    return("L line is shorted to ground");
	case ERROR_NO_RESP_RECV_TO_INIT: 
	    return("No response received to init command");
	case ERROR_IMPROPER_SYNC_RECV: 
	    return("Improper sync byte received");
	case ERROR_IMPROPER_INV_ADDR_RECV: 
	    return("Improper inverse address byte received");
	case ERROR_IMPROPER_START_COMM_POS_RESP: 
	    return("Improper start communications positive response received");
	case ERROR_BAD_START_COMM_POS_RESP_CKS:
	    return("Bad start communications positive response checksum");
	case ERROR_INVALID_CHECKSUM_FROM_PC: 
	    return("Invalid checksum from PC received");
	case ERROR_INVALID_COMMAND_FROM_PC: 
	    return("Invalid command from PC received");
	case ERROR_RECEIVE_BUFFER_OVERFLOW: 
	    return("Receive buffer overflow");
	case ERROR_TRANSMIT_BUFFER_OVERFLOW:
	    return("Transmit buffer overflow");
	case ERROR_NULL: 
	    return("No error");
	case ERROR_ROM_CHECKSUM_ERROR: 
	    return("Internal ROM checksum error; contact Silicon Engines");
	default:
	    return("Unknown error code");
    }
}

