/* NISSAN.C
 * Nissan Parameter Handler
 * 03/18/02 Prune
 */
#define NISSAN_MAIN
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "nissan.h"
#include "isoapp.h"
#include "rs232.h"

extern unsigned char EnableTesterPresent;

char * GetNamedParameter(ParmEnum x)
{
    switch (x) {
	case PARAM_ID_OPTION: return("Identification Option");
    	case PARAM_TRANSMISSION_MODE: return("Transmission Mode");
    	case PARAM_MAX_NUM_OF_RESP_TO_SEND: return("Max Number of Responses to Send");
	case PARAM_RECORD_LOCAL_ID: return("Record Local ID");
    	case PARAM_RECORD_COMMON_ID_HIGH_BYTE: return("Record Common ID High Byte");
	case PARAM_RECORD_COMMON_ID_LOW_BYTE: return("Record Common ID Low Byte");
    	case PARAM_WRITE_DATA_LOCAL_ID: return("Write Data Local ID");
	case PARAM_CONTROL_OPTION_1: return("Control Option 1");
    	case PARAM_CONTROL_OPTION_2: return("Control Option 2");
	case PARAM_RECORD_ID: return("Record ID");
	case PARAM_IO_LOCAL_ID: return("Input Output Local ID");
    	case PARAM_IO_CTRL_OPTION_1_HIGH_BYTE: return("Input Output Control Option 1 High Byte");
	case PARAM_IO_CTRL_OPTION_1_LOW_BYTE: return("Input Output Control Option 1 Low Byte");
	case PARAM_IO_CTRL_OPTION_2: return("Input Output Control Option 2");
	case PARAM_VALVE_CTRL_OPTION_1: return("Valve Control Option 1");
	case PARAM_VALVE_CTRL_OPTION_2: return("Valve Control Option 2");
    	case PARAM_EEPROM_DUMP_CTRL_OPTION_1: return("EEPROM Dump Control Option 1");
        default: return("Unknown Parameter");
   }
}


void ValveActivation(void)
{
    printf("\nSending Valve Activation:");
    Checksum=0;
    PrintAndWrite(HEADER(4));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0xB1);
    PrintAndWrite(0x04);
    PrintAndWrite(Parameter[PARAM_VALVE_CTRL_OPTION_1]);
    PrintAndWrite(Parameter[PARAM_VALVE_CTRL_OPTION_2]);
    PrintAndWrite(Checksum);
    printf("...");
}

void EEPROMDump(void)
{
    printf("\nSending EEPROMDump:");
    Checksum=0;
    PrintAndWrite(HEADER(4));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0xB1);
    PrintAndWrite(0x02);
    PrintAndWrite(Parameter[PARAM_EEPROM_DUMP_CTRL_OPTION_1]);
    PrintAndWrite(0x00);
    PrintAndWrite(Checksum);
    printf("...");
}

void ReadFreezeFrameData(void)
{
    printf("\nSending ReadFreezeFrameData:");
    Checksum=0;
    PrintAndWrite(HEADER(4));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0x12);
    PrintAndWrite(0x01);
    PrintAndWrite(0x01);
    PrintAndWrite(Parameter[PARAM_RECORD_ID]);
    PrintAndWrite(Checksum);
    printf("...\n");
}

void ReadSoftwareVersion(void)
{
    printf("\nSending ReadSoftwareVersion:");
    Checksum=0;
    PrintAndWrite(HEADER(4));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0xB1);
    PrintAndWrite(0x03);
    PrintAndWrite(0x00);
    PrintAndWrite(0x00);
    PrintAndWrite(Checksum);
    printf("...\n");
}

void ClearDiagnosticInformation(void)
{
    printf("\nSending ClearDiagnosticInformation:");
    Checksum=0;
    PrintAndWrite(HEADER(4));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0xB1);
    PrintAndWrite(0x01);
    PrintAndWrite(0x00);
    PrintAndWrite(0x00);
    PrintAndWrite(Checksum);
    printf("...\n");
}

void ClearDiagnosticTroubleCodes(void)
{
    printf("\nSending ClearDiagnosticTroubleCodes:");
    Checksum=0;
    PrintAndWrite(HEADER(1));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0x14);
    PrintAndWrite(Checksum);
    printf("...\n");
}
 
void ReadECUIdentification(void)
{
    printf("\nSending ReadECUIdentification:");
    Checksum=0;
    PrintAndWrite(HEADER(2));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0x1A);
    PrintAndWrite(Parameter[PARAM_ID_OPTION]);
    PrintAndWrite(Checksum);
    printf("...");    
}
 
void ReadDataByLocalIdentifier(void)
{
    printf("\nSending ReadDataByLocalIdentifier:");
    Checksum=0;
    PrintAndWrite(HEADER(4));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0x21);
    PrintAndWrite(Parameter[PARAM_RECORD_LOCAL_ID]);
    PrintAndWrite(Parameter[PARAM_TRANSMISSION_MODE]);
    PrintAndWrite(Parameter[PARAM_MAX_NUM_OF_RESP_TO_SEND]);
    PrintAndWrite(Checksum);
    printf("...\n");
}

void ReadDataByCommonIdentifier(void)
{
    printf("\nSending ReadDataByCommonIdentifier:");
    Checksum=0;
    PrintAndWrite(HEADER(5));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0x22);
    PrintAndWrite(Parameter[PARAM_RECORD_COMMON_ID_HIGH_BYTE]);
    PrintAndWrite(Parameter[PARAM_RECORD_COMMON_ID_LOW_BYTE]);
    PrintAndWrite(Parameter[PARAM_TRANSMISSION_MODE]);
    PrintAndWrite(Parameter[PARAM_MAX_NUM_OF_RESP_TO_SEND]);
    PrintAndWrite(Checksum);
    printf("...\n");
}
 
void InputOutputControlByLocalIdentifier(void)
{
    printf("\nSending InputOutputControlByLocalIdentifier:");
    Checksum=0;
    if (Parameter[PARAM_IO_LOCAL_ID]>=0x60) {
        PrintAndWrite(HEADER(5));
    } else {
        PrintAndWrite(HEADER(4));
    }
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0x30);
    PrintAndWrite(Parameter[PARAM_IO_LOCAL_ID]);
    PrintAndWrite(Parameter[PARAM_IO_CTRL_OPTION_1_HIGH_BYTE]);
    if (Parameter[PARAM_IO_LOCAL_ID]>=0x60) {
  	PrintAndWrite(Parameter[PARAM_IO_CTRL_OPTION_1_LOW_BYTE]);
    }
    PrintAndWrite(Parameter[PARAM_IO_CTRL_OPTION_2]);
    PrintAndWrite(Checksum);
    printf("...");
}

void WriteDataByLocalIdentifier(void)
{
    printf("\nSending WriteDataByLocalIdentifier:");
    Checksum=0;
    PrintAndWrite(HEADER(4));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0x3B);
    PrintAndWrite(Parameter[PARAM_WRITE_DATA_LOCAL_ID]);
    PrintAndWrite(Parameter[PARAM_CONTROL_OPTION_1]);
    PrintAndWrite(Parameter[PARAM_CONTROL_OPTION_2]);
    PrintAndWrite(Checksum);
    printf("...");
}
 
void ReadDiagnosticTroubleCodesWithOptionByte(void)
{
    printf("This service is not defined in spec Application Layer Specification"
	" A T E N 462-20.08 Sheet 151\n");
}

 
void ReadTimeOfDiagnosticTroubleCodes(void)
{
    printf("\nSending ReadTimeOfDiagnosticTroubleCodes:");
    Checksum=0;
    PrintAndWrite(HEADER(1));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0xA3);
    PrintAndWrite(Checksum);
}

 
void SimplyDynamicallyDefineLocalID(void)
{
    printf("\nSending SimplyDynamicallyDefineLocalID:");
    Checksum=0;
    PrintAndWrite(HEADER(5));
    PrintAndWrite(Target);
    PrintAndWrite(Source);
    PrintAndWrite(0xAC);
    PrintAndWrite(0x81);
    PrintAndWrite(0x02);
    PrintAndWrite(Parameter[PARAM_RECORD_COMMON_ID_HIGH_BYTE]);
    PrintAndWrite(Parameter[PARAM_RECORD_COMMON_ID_LOW_BYTE]);
    PrintAndWrite(Checksum);
    printf("...\n");
}

void TesterPresent(void)
/* to be done silently */
{
    if ((MyReadCharPending()<0)&&(EnableTesterPresent)) {
        Checksum=0;
        MyWriteChar(HEADER(2)); Checksum+=HEADER(2);
        MyWriteChar(Target); Checksum+=Target;
        MyWriteChar(Source); Checksum+=Source;
        MyWriteChar(0x3E); Checksum+=0x3E;
        MyWriteChar(0x01); Checksum+=0x01;
        MyWriteChar(Checksum); 
    }
}

void LoadParameters(void)
{
FILE *nfp;
unsigned char x;
unsigned char string[16];
    if ((nfp=fopen("NISSAN.PRM","rt"))==NULL) {
	perror("NISSAN.PRM");
	for (x=0; x<NUMBER_OF_PARAMETERS; x++) {
	    Parameter[x]=0;
	}
    } else {
	for (x=0; x<NUMBER_OF_PARAMETERS; x++) {
	    fgets(string,15,nfp);
	    Parameter[x]=atoi(string);
	}
	fclose(nfp);
    }
}

void SaveParameters(void)
{
FILE *nfp;
unsigned char x;
    if ((nfp=fopen("NISSAN.PRM","wt"))==NULL) {
	perror("NISSAN.PRM");
    } else {
	for (x=0; x<NUMBER_OF_PARAMETERS; x++) {
	    fprintf(nfp,"%u\n",Parameter[x]);
	}
	fclose(nfp);
    }
}

