/* RS232.C
 * RS232 module 
 * 3/15/02 Prune thanx to Jeff Hunsinger for many of these routines
 */
#define RS232_MAIN

#include <dos.h>
#include <conio.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include "rs232.h"

const int com[4]={0x3f8,0x2f8,0x3e8,0x2e8};   /* Addresses of serial port */

extern void clean(int x); /* exit if ESC pressed */

int detect_UART(unsigned baseaddr)
{
   /* this function returns 0 if no UART is installed. */
   /* 1: 8250, 2: 16450 or 8250 with scratch reg., 3: 16550, 4: 16550A */
   int x,olddata;

   /* check if a UART is present anyway */
   olddata=inportb(baseaddr+4);
   outportb(baseaddr+4,0x10);
   if ((inportb(baseaddr+6)&0xf0)) return 0;
   outportb(baseaddr+4,0x1f);
   if ((inportb(baseaddr+6)&0xf0)!=0xf0) return 0;
   outportb(baseaddr+4,olddata);
   /* next thing to do is look for the scratch register */
   olddata=inportb(baseaddr+7);
   outportb(baseaddr+7,0x55);
   if (inportb(baseaddr+7)!=0x55) return 1;
   outportb(baseaddr+7,0xAA);
   if (inportb(baseaddr+7)!=0xAA) return 1;
   outportb(baseaddr+7,olddata); /* we don't need to restore it if it's not there */
   /* then check if there's a FIFO */
   outportb(baseaddr+2,1);
   x=inportb(baseaddr+2);
   outportb(baseaddr+2,0x0);
   if ((x&0x80)==0) return 2;
   if ((x&0x40)==0) return 3;
   return 4;
}

void handshake(const unsigned char settings)
{
    outportb(com[comport]+4,settings);
}

char InitComm(int port,unsigned long baudrate)
{
    comport=port;
    outportb(com[comport]+3,0x80); /* No parity, 8 bits, set DLAB */
    outportb(com[comport],(((115200/baudrate)&0xFF)));
    outportb(com[comport]+1,(((115200/baudrate)>>8)));
    outportb(com[comport]+3,0x03);           /* Turn off DLAB */
    inportb(com[comport]);                   /* Clear buffer */
    inportb(com[comport]+6);
    /* outportb(com[comport]+2,0); *//* 7/9/03 */
    switch(detect_UART(com[comport])) {
      case 0:
         printf("No UART detected at COM%d\n",comport+1);
         break;
       case 1:
         break;
       case 2:
         break;
       case 3:
         break;
       case 4:
       default:
         outportb(com[comport]+2,0x07);               /* Turn on FIFO */
         break;
    }
    handshake(RTS|DTR);
    return(1);
}

int MyWriteChar(int c)
{
    while ((!(inportb(com[comport]+5) & TXRDY))||
	(!(inportb(com[comport]+5)& TXCPLTE))|| /* tried 7/9/03 */
	(!(inportb(com[comport]+6) & CTS))) {      /* Wait for Tx Rdy & CTS */
        if (kbhit()) {
            if (getch()==27) clean(-3);
	}
    }
    outportb(com[comport],c);                      /* Send byte */
    return(0);
}

int MyReadCharPending(void)
{
    if (!(inportb(com[comport]+5) & RXRDY)) return(-1);
    return(0);
}


int MyReadChar(void)
{
    if (!(inportb(com[comport]+5) & RXRDY)) return(-1);
    return(inportb(com[comport]));                        /* Read data reg */
}

int MyReadCharTimed(long milliseconds)
{
time_t x;
    time(&x);
    while (1) {
        if ((time(NULL)-x)>=((milliseconds+999)/1000)) break;
        if (inportb(com[comport]+5) & RXRDY) {
    	    return(inportb(com[comport]));                    /* Read data reg */
	}
    }
    return(-1);
}

int MyClearRXBuffer(void)
{
    while (inportb(com[comport]+5) & RXRDY) {
        if (kbhit()) {
            if (getch()==27) clean(-3);
	}
        inportb(com[comport]);                        /* Read data reg */
    }
    return(0);
}

void clean_rs232(void) 
{
 /* no extra work is required for this app */ 
}

int modemstatus(void)
{
    return(inportb(com[comport]+6));
}

